@@ -74,7 +74,7 @@ mt_main_generate_motion_event (GdkScreen *screen, gint x, gint y)
 static void
 mt_main_generate_button_event (MTClosure *mt,
 			       guint      button,
-			       gboolean   type,
+			       gint       type,
 			       gulong     delay)
 {
     gdk_error_trap_push ();
@@ -99,7 +99,7 @@ mt_main_generate_button_event (MTClosure *mt,
 	    XTestFakeButtonEvent (mt->xtst_display,
 				  button, False, delay);
 	    XTestFakeButtonEvent (mt->xtst_display,
-				  button, True, CurrentTime);
+				  button, True, delay);
 	    XTestFakeButtonEvent (mt->xtst_display,
 				  button, False, delay);
 	    break;
@@ -420,6 +420,13 @@ global_motion_event (MtListener *listener,
 {
     MTClosure *mt = data;
 
+    if (mt_timer_is_running (mt->delay_timer)) {
+	if (!below_threshold (mt, event->x, event->y)) {
+	    mt_timer_stop (mt->delay_timer);
+	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
+	}
+    }
+
     if (mt->dwell_enabled) {
 	if (!below_threshold (mt, event->x, event->y) &&
 	    !mt->dwell_gesture_started) {
@@ -441,13 +448,6 @@ global_motion_event (MtListener *listener,
 	    mt->y_old = event->y;
 	}
     }
-
-    if (mt_timer_is_running (mt->delay_timer)) {
-	if (!below_threshold (mt, event->x, event->y)) {
-	    mt_timer_stop (mt->delay_timer);
-	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
-	}
-    }
 }
 
 static void
